-- GLOBAL --
SAM = select(2, ...); 
-- MODULE --
local _G = getfenv(0);
setmetatable(SAM, {__index = _G});
setfenv(1, SAM);
-- LOCAL --
title		= "StopAddonMessage"
folder		= title;
version		= GetAddOnMetadata(folder, "X-Curse-Packaged-Version") or "";
titleFull	= title.." "..version

coreFrame = CreateFrame("Frame", nil, UIParent);
core	= LibStub("AceAddon-3.0"):NewAddon(coreFrame, title, "AceConsole-3.0", "AceHook-3.0") -- 
local L = LibStub("AceLocale-3.0"):GetLocale(title, true)
P = {}; --db.profile

lastOnUpdate = 0;

directory	= "Interface\\Addons\\"..folder.."\\";

--Globals
local GAMEOPTIONS_MENU_ = _G.GAMEOPTIONS_MENU;
local ENABLE_ = _G.ENABLE;
local pairs_ = _G.pairs;
local InterfaceOptionsFrame_OpenToCategory_ = _G.InterfaceOptionsFrame_OpenToCategory;
local tostring_ = _G.tostring;
local table_getn = _G.table.getn;
local print_ = _G.print;
local GetFramesRegisteredForEvent_ = _G.GetFramesRegisteredForEvent;

-- Default Settings
defaultSettings = {profile = {}}
defaultSettings.profile.defaultOutgoingAction	= 0;--0=allow, 1=deny
defaultSettings.profile.defaultIncomingAction	= 0;--0=allow, 1=deny
defaultSettings.profile.printOutgoingMessages		= 4;--new
defaultSettings.profile.printIncomingMessages		= 0;--off
defaultSettings.profile.chatFrame	= 1;--main chatframe.

defaultSettings.profile.prefixActions	= {};

-- Echo Strings
local strDebugFrom		= "|cffffff00[%s]|r" --Yellow function name. help pinpoint where the debug msg is from.
local strWhiteBar		= "|cffffff00 || |r" -- a white bar to seperate the debug info.
local greenText			= "|cff00ff00%s|r";
local redText			= "|cffff0000%s|r"
local yellowText		= "|cffffff00%s|r";
local grayText			= "|cff7f7f7f%s|r"

local coreUI;
local prefixUI;
local newPrefixes = {};

local userName;

-- Ace3 Functions
function core:OnInitialize()
	db = LibStub("AceDB-3.0"):New("SAM_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")

	P = db.profile;

	core:RegisterChatCommand("sam", "MySlashProcessorFunc")
	
	
	BuildPrefixUIs()
	local config = LibStub("AceConfig-3.0");
	local dialog = LibStub("AceConfigDialog-3.0");
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreUI = dialog:AddToBlizOptions(title, "SAM "..version)
	
	config:RegisterOptionsTable(title.."Prefix", PrefixOptionsTable)
	prefixUI = dialog:AddToBlizOptions(title.."Prefix", L["Prefixes"], "SAM "..version)
	
	
	
	--reset options on old version.
	if version ~= "" then
		if not P.lastVersion then
			P.prefixActions = {};
			P.lastVersion = version
		end
	end
end

function core:OnEnable()
	--make all options clickable.
	for name, data in pairs_(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = false;
		end
	end
	
	userName = UnitName("player");
	
	HookOnEventFrames( GetFramesRegisteredForEvent_("CHAT_MSG_ADDON") )
	
	self:RawHook("SendAddonMessage", true)
	BuildPrefixUIs()
end

----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /att function brings up the UI options.	--
----------------------------------------------
	InterfaceOptionsFrame_OpenToCategory_(prefixUI)
	InterfaceOptionsFrame_OpenToCategory_(coreUI)
end


------------------------------------------------------------------
function HookOnEventFrames ( ... )								--	/script SAM.HookOnEventFrames( GetFramesRegisteredForEvent("CHAT_MSG_ADDON") )
-- Raw hook frames so we can intercept their incoming messages.	--
------------------------------------------------------------------
    for i = 1, select( "#", ... ) do
		local frame = select(i, ...)
		core:RawHookScript(frame, "OnEvent", "OnEventHook")
    end
end


----------------------------------------------------------------------
function core:OnEventHook(frame, event, ...)						--
-- Our incoming message hook. We intercept incoming messages here.	--
----------------------------------------------------------------------
	if event == "CHAT_MSG_ADDON" then
		local prefix, message, distribution, sender = ...;
		if sender ~= userName then --no need to block our own messages right?
			if P.prefixActions[prefix] then
				if P.prefixActions[prefix].incomingAction == 0 then --allow
					P.prefixActions[prefix].incomingAllowed = P.prefixActions[prefix].incomingAllowed + 1;
					
					PrintIncomingAddonMessage(L["Allowed"], false, ...)
				else
					P.prefixActions[prefix].incomingDenied = P.prefixActions[prefix].incomingDenied + 1;
					PrintIncomingAddonMessage(L["Blocked"], false, ...)
					return;
				end
			else
				--new prefix, add it to options screen.
				newPrefixes[prefix] = true;
				P.prefixActions[prefix] = {incomingDenied=0,incomingAllowed=0, outgoingDenied=0,outgoingAllowed=0}
				
				P.prefixActions[prefix].incomingAction = P.defaultIncomingAction;
				P.prefixActions[prefix].outgoingAction = P.defaultOutgoingAction;
				
				
				if P.defaultIncomingAction == 0 then
					P.prefixActions[prefix].incomingAllowed = 1;
		
					PrintIncomingAddonMessage(L["Allowed"], true, ...)
					
					BuildPrefixUIs()
				else
					P.prefixActions[prefix].incomingDenied = 1;
					PrintIncomingAddonMessage(L["Blocked"], true, ...)
					
					BuildPrefixUIs()
					return;
				end
			end
		end
	end
	
	--send message to addon's event handler.
	self.hooks[frame].OnEvent(frame, event, ...)
end


--	/script SendAddonMessage("123", "blaw","GUILD")

--	/script SAM.P.prefixActions = {}

--------------------------------------------------------------------------------------
function core:SendAddonMessage(...)													--
-- Our send addon message hook. We check what prefix is being sent and block it.	--
--------------------------------------------------------------------------------------
	local prefix, text, type, target = ...;
	
	if P.prefixActions[prefix] then
		if P.prefixActions[prefix].outgoingAction == 0 then --allow
			P.prefixActions[prefix].outgoingAllowed = P.prefixActions[prefix].outgoingAllowed + 1;
			
			PrintOutgoingAddonMessage(L["Allowed"], false, ...)
		else
			P.prefixActions[prefix].outgoingDenied = P.prefixActions[prefix].outgoingDenied + 1;
			PrintOutgoingAddonMessage(L["Blocked"], false, ...)
			return;
		end
	else
		newPrefixes[prefix] = true;
		P.prefixActions[prefix] = {incomingDenied=0,incomingAllowed=0, outgoingDenied=0,outgoingAllowed=0}
		P.prefixActions[prefix].incomingAction = P.defaultIncomingAction;
		P.prefixActions[prefix].outgoingAction = P.defaultOutgoingAction;
		
		if P.defaultOutgoingAction == 0 then
			P.prefixActions[prefix].outgoingAllowed = 1;

			PrintOutgoingAddonMessage(L["Allowed"], true, ...)
			
			BuildPrefixUIs()
		else
			P.prefixActions[prefix].outgoingDenied = 1;


			PrintOutgoingAddonMessage(L["Blocked"], true, ...)
			
			BuildPrefixUIs()
			return;
		end
	end
	
	core.hooks.SendAddonMessage(...)
end

--	/script SAM.P.prefixActions = {};


function core:OnDisable()
	--make all the options unclickable except for the enable button.
	for name, data in pairs_(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = true;
		end
	end
end



----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset addon.						--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end



------------------------------
function Debug(from, ...)	--	/script DA.Debug("hey","bye")
-- simple print function.	--
------------------------------
	if version ~= "" then--version's from curse, user doesn't need to see debug msgs.
		return;
	end
	local tbl  = {...}
	local msg = tostring_(tbl[1])
	for i=2,table_getn(tbl) do 
		msg = msg..strWhiteBar..tostring_(tbl[i]);
	end
	print_(strDebugFrom:format(from).." "..msg)
end

function PrintOutgoingAddonMessage(action, new, ...)
	local prefix, message, distribution, target  = ...;
	
	if not P.prefixActions[prefix].printMsgs or P.prefixActions[prefix].printMsgs == false then
		if P.printOutgoingMessages == 0 then
			return;
		elseif P.printOutgoingMessages == 2 and action ~= L["Allowed"] then
			return;
		elseif P.printOutgoingMessages == 3 and action ~= L["Blocked"] then
			return;
		elseif P.printOutgoingMessages == 4 and new ~= true then
			return;
		end
	end
	
	local cf = _G["ChatFrame"..P.chatFrame];
	if cf then
		if distribution == "WHISPER" then
			cf:AddMessage("|cffffffff"..L["Outgoing"].." "..action.."|r> |cffff0000"..prefix.."|r || |cff00ff00"..distribution.."|r || |cff00ffff"..target.."|r || "..message,.5,.5,.5)
		else
			cf:AddMessage("|cffffffff"..L["Outgoing"].." "..action.."|r> |cffff0000"..prefix.."|r || |cff00ff00"..distribution.."|r || "..message,.5,.5,.5)
		end
	end
end

function PrintIncomingAddonMessage(action, new, ...)
	local prefix, message, distribution, sender  = ...;
	
	if not P.prefixActions[prefix].printMsgs or P.prefixActions[prefix].printMsgs == false then
		if P.printIncomingMessages == 0 then
			return;
		elseif P.printIncomingMessages == 2 and action ~= L["Allowed"] then
			return;
		elseif P.printIncomingMessages == 3 and action ~= L["Blocked"] then
			return;
		elseif P.printIncomingMessages == 4 and new ~= true then
			return;
		end
	end
	
	local cf = _G["ChatFrame"..P.chatFrame];
	if cf then
--~ 		if distribution == "WHISPER" then
			cf:AddMessage("|cffffffff"..L["Incoming"].." "..action.."|r> |cffff0000"..prefix.."|r || |cff00ff00"..distribution.."|r || |cff00ffff"..sender.."|r || "..message,.5,.5,.5)
--~ 		else
--~ 			cf:AddMessage("|cffffffff"..flow.." "..action.."|r> |cffff0000"..prefix.."|r || |cff00ff00"..distribution.."|r || "..message,.5,.5,.5)
--~ 		end
	end
end




----------------
--     UI     --
----------------

PrefixOptionsTable = {
	name = titleFull,
	type = "group",
	args = {},
}


CoreOptionsTable = {
	name = titleFull,
	type = "group",
	childGroups = "tab",

	args = {
		core={
			name = GAMEOPTIONS_MENU_,
			type = "group",
			order = 1,
			args={}
		},
	},
}




CoreOptionsTable.args.core.args.enable = {
	type = "toggle",	order	= 1,
	name	= ENABLE_,
	desc	= L["Enables / Disables the addon"],
	set = function(info,val) 
		if val == true then
			core:Enable();
		else
			core:Disable();
		end
	end,
	get = function(info) return core:IsEnabled() end
}

CoreOptionsTable.args.core.args.chatFrame = {
	name = L["Chat Frame"],
	order = 2,
	desc = L["Chatframe to print messages to"],
	type = "select",
	
	values = function()
		local tbl = {};
		
		for i=1, 10 do 
			table.insert(tbl,(_G["ChatFrame"..i] and _G["ChatFrame"..i].name) or "ChatFrame"..i)
		end
	
		return tbl
		
	end,
	set = function(info,val) 
		P.chatFrame = val;
	end,
	get = function(info) return P.chatFrame end
}


CoreOptionsTable.args.core.args.incomingHeader = {
	name	= L["Incoming"],
	order	= 10,
	type = "header",
}

CoreOptionsTable.args.core.args.defaultIncomingAction = {
	name = L["Default Action"],
	order = 11,
	desc = L["Default action for new incoming prefixes"],
	type = "select",
	
	values = function()
		return {[0]=greenText:format(L["Allow"]), [1]=redText:format(L["Block"])}
	end,
	set = function(info,val) 
		P.defaultIncomingAction = val;
	end,
	get = function(info) return P.defaultIncomingAction end
}

CoreOptionsTable.args.core.args.printIncomingMessages = {
	name = L["Print Messages"],
	order = 12,
	desc = L["Print your incoming messages to a chatframe."],
	type = "select",
	
	values = function()
		return {
			[0] = L["Off"],
			[1] = L["All"],
			[2] = L["Allowed"],
			[3] = L["Blocked"],
			[4] = L["New"],
		}
		
	end,
	set = function(info,val) 
		P.printIncomingMessages = val;
	end,
	get = function(info) return P.printIncomingMessages end
}



CoreOptionsTable.args.core.args.outgoingHeader = {
	name	= L["Outgoing"],
	order	= 20,
	type = "header",
}


CoreOptionsTable.args.core.args.defaultOutgoingAction = {
	name = L["Default Action"],
	order = 21,
	desc = L["Default action for new outgoing prefixes"],
	type = "select",
	
	values = function()
		return {[0]=greenText:format(L["Allow"]), [1]=redText:format(L["Block"])}
	end,
	set = function(info,val) 
		P.defaultOutgoingAction = val;
	end,
	get = function(info) return P.defaultOutgoingAction end
}



CoreOptionsTable.args.core.args.printOutgoingMessages = {
	name = L["Print Messages"],
	order = 22,
	desc = L["Print your outgoing messages to a chatframe."],
	type = "select",
	
	values = function()
		return {
			[0] = L["Off"],
			[1] = L["All"],
			[2] = L["Allowed"],
			[3] = L["Blocked"],
			[4] = L["New"],
		}
		
	end,
	set = function(info,val) 
		P.printOutgoingMessages = val;
	end,
	get = function(info) return P.printOutgoingMessages end
}





function BuildPrefixUIs()
	PrefixOptionsTable.args = {}; --reset

	local list = {};
	for prefix, data in pairs(P.prefixActions) do 
		table.insert(list, prefix)
	end
	
	table.sort(list, function(a,b) 
		if(a and b) then 
			return a < b;
		end 
	end)
	
	local prefix;
	local pColoured;
	for i=1, table.getn(list) do 
		prefix = list[i];
		
		if P.prefixActions[prefix].outgoingAction == 0 and P.prefixActions[prefix].incomingAction == 0 then
			pColoured = greenText:format(prefix);
		elseif P.prefixActions[prefix].outgoingAction == 1 and P.prefixActions[prefix].incomingAction == 1 then
			pColoured = redText:format(prefix);
		else
			pColoured = yellowText:format(prefix);
		end
		
--~ 		pColoured = prefix;
		
		if newPrefixes[prefix] then
			pColoured = pColoured .. " "..grayText:format(L["New"]);
		end
		
		
		PrefixOptionsTable.args[prefix] = {
			type	= "group",
			name	= pColoured,
			desc	= (knownPrefixes[prefix] or L["Unknown"]),
			order = i,
			args={}
		}
		
		
		PrefixOptionsTable.args[prefix].args.addon = {
			type = "description",
			name = L["Addon: "]..(knownPrefixes[prefix] or L["Unknown"]),
			order = 1,
		}

		PrefixOptionsTable.args[prefix].args.incomingAction = {
			name = L["Incoming"].." "..L["Action"],
			order = 4,
			desc = L["What to do when message is received."],
			type = "select",
			
			values = function()
				return {[0]=greenText:format(L["Allow"]), [1]=redText:format(L["Block"])}
			end,
			set = function(info,val) 
				P.prefixActions[info[1]].incomingAction = val;
				BuildPrefixUIs()
			end,
			get = function(info) return P.prefixActions[info[1]].incomingAction end
		}
		
		PrefixOptionsTable.args[prefix].args.outgoingAction = {
			name = L["Outgoing"].." "..L["Action"],
			order = 5,
			desc = L["What to do when message is sent."],
			type = "select",
			
			values = function()
				return {[0]=greenText:format(L["Allow"]), [1]=redText:format(L["Block"])}
			end,
			set = function(info,val) 
				P.prefixActions[info[1]].outgoingAction = val;
				BuildPrefixUIs()
			end,
			get = function(info) return P.prefixActions[info[1]].outgoingAction end
		}

		PrefixOptionsTable.args[prefix].args.incomingCount = {
			type = "description",
			name = L["Incoming"]..": "..greenText:format((P.prefixActions[prefix].incomingAllowed or 0)).." :: "..redText:format((P.prefixActions[prefix].incomingDenied or 0)),
			order = 6,
		}
		
		PrefixOptionsTable.args[prefix].args.outgoingCount = {
			type = "description",
			name = L["Outgoing"]..": "..greenText:format((P.prefixActions[prefix].outgoingAllowed or 0)).." :: "..redText:format((P.prefixActions[prefix].outgoingDenied or 0)),
			order = 7,
		}
		


		PrefixOptionsTable.args[prefix].args.printMsgs = {
			type = "toggle",	order	= 8,
			name	= L["Print Msg"],
			desc	= L["Print this prefix's messages to chat frame."],
			set = function(info,val) 
				P.prefixActions[info[1]].printMsgs = val;
			end,
			get = function(info) return P.prefixActions[info[1]].printMsgs end
		}	
		
		
	end
	
end